'' +--------------------------------------------------------------------------+
'' | Cluso's "AutoDetect P2D2 or P2EVAL PCB"  (c)2019 "Cluso99" (Ray Rodrick) |
'' |                     MIT License                                          |
'' +--------------------------------------------------------------------------+
'' RR20190916  001  first release
''             004  auto calc clock params
'' RR20190917  005  check clkmode in hub and if !0 disable pll first
''                  write clkmode and clkfreq to hub $00014 and $00018
''             006  fix  _XDIVP = (_CLOCKFREQ > 180_000_000) ? 1 : 2 
''                       _XMUL  = _CLOCKFREQ * _XDIVP / _BASEFREQ    
'' RR20190918  007  store BAUD in hub $1C
''                  locals bitper,clkfreq,clkmode,baud --> bitperx,clkfreqx,clkmodex,baudx
''             008  load clkfreq as constant in case overwritten by loader!

CON
{{-----------------------------------------------------------------------------
'recalculate sysclock hertz using cordic
                qmul    xtalmul, ##(XTALFREQ / XDIV / XDIVP)    'integer component of pre-divided crystal frequency
                mov     pa, xtalmul
                mul     pa, ##round((float(XDIV*XDIVP)+0.5) * (float(XTALFREQ)/float(XDIV)/float(XDIVP) - float(XTALFREQ/XDIV/XDIVP)))
                qdiv    pa, ##(XDIV * XDIVP)            'fractional component of pre-divided crystal frequency
                getqx   clk_freq                        'result of integer component
                getqx   pa                              'result of fractional component
                add     clk_freq, pa                    'de-error the integer rounding
-----------------------------------------------------------------------------}}

''============================[ CON ]============================================================
CON
'+-------[ Select desired clock frequency ]------------------------------------+  
  _CLOCKFREQ     = 160_000_000                                   ' clock frequency
  _BASEFREQ      = 4_000_000                                     ' common base freq after XDIV
'+-------[ Crystal Parameters ]------------------------------------------------+  
  _XTALFREQ2     = 12_000_000                                    ' P2D2   crystal frequency
  _XOSC2         = %01                                   'OSC    ' %00=OFF, %01=OSC, %10=15pF, %11=30pF
'+-----------------------------------------------------------------------------+
  _XTALFREQV     = 20_000_000                                    ' P2EVAL crystal frequency
  _XOSCV         = %10                                   '15pF   ' %00=OFF, %01=OSC, %10=15pF, %11=30pF
'+-----------------------------------------------------------------------------+
' calculate values (auto)...
  _XDIV2         = _XTALFREQ2 / _BASEFREQ                        ' P2D2   crystal divider
  _XDIVV         = _XTALFREQV / _BASEFREQ                        ' P2EVAL crystal divider
  _XDIVP         = (_CLOCKFREQ > 180_000_000) ? 1 : 2            ' if >180MHz /1 else /2
  _XMUL          = _CLOCKFREQ * _XDIVP / _BASEFREQ               ' clockfreq = crystal / div * mul * divp
  _XPPPP         = ((_XDIVP>>1) + 15) & $F                       ' 1->15, 2->0, 4->1, 6->2...30->14
  _CLKMODE2      = 1<<24 + (_XDIV2-1)<<18 + (_XMUL-1)<<8 + _XPPPP<<4 + _XOSC2<<2  ' %0000_000e_dddddd_mmmmmmmmmm_pppp_cc_00  ' P2D2   setup
  _CLKMODEV      = 1<<24 + (_XDIVV-1)<<18 + (_XMUL-1)<<8 + _XPPPP<<4 + _XOSCV<<2  ' %0000_000e_dddddd_mmmmmmmmmm_pppp_cc_00  ' P2EVAL setup
  _XSEL          = %11    ' %00=rcfast(20+MHz), %01=rcslow(~20KHz), %10=XI(5ms), %11=XI+PLL(10ms)        ......._...._.._11  ' XI+PLL
'+-----------------------------------------------------------------------------+

'+-------[ Serial Setup ]------------------------------------------------------+ 
  _BAUD         = 115_200
  _BITPER       = (_CLOCKFREQ / _BAUD) << 16 + 7          ' 115200 baud, 8 bits
  _txmode       = %0000_0000_000_0000000000000_01_11110_0 'async tx mode, output enabled for smart output
  _rxmode       = %0000_0000_000_0000000000000_00_11111_0 'async rx mode, input  enabled for smart input
'+-----------------------------------------------------------------------------+
  rx_pin        = 63            ' pin serial receiver
  tx_pin        = 62            ' pin serial transmitter
'+-----------------------------------------------------------------------------+
  pincfg        = 25            ' P2D2 pin has 10K pull-down, P2EVAL has nc
  LED_p2d2      = 0             ' P2D2  Green LED 1=ON
  LED_p2eval    = 56            ' P2EVAL Blue LED 0=ON

CON
'' +--------------------------------------------------------------------------+
''   P2 ROM SERIAL ROUTINES (HUBEXEC)
'' +--------------------------------------------------------------------------+
  _SerialInit      = $fcab8     ' Serial Initialise     (lmm_x & lmm_bufad must be set first)
  _HubTxString     = $fcba4     ' Sends $0 terminated string at lmm_p address
'' +--------------------------------------------------------------------------+
''  COG VARIABLES $1E0-$1EF - ROM MONITOR
'' +--------------------------------------------------------------------------+
  lmm_x         = $1e0          ' parameter passed to/from LMM routine (typically a value)
  lmm_f         = $1e1          ' parameter passed to      LMM routine (function options; returns unchanged)
  lmm_p         = $1e2          ' parameter passed to/from LMM routine (typically a hub/cog ptr/addr)
  lmm_p2        = $1e3          ' parameter passed to/from LMM routine (typically a 2nd hub/cog address)
  lmm_c         = $1e4          ' parameter passed to/from LMM routine (typically a count)
  'reserved     = $1e5-$1ee     ' workareas
  lmm_bufad     = $1ef          ' _HubRxString
'' +--------------------------------------------------------------------------+
  _HUBBUF       = $FC000        ' overwrite Booter for buffer

DAT           org       0                               'hubaddr if first loaded into hub $0

entry         jmp       #begin                          '00000 skip over hub
pcb           long      0                               '00004  0=P2D2, 1=P2EVAL
clkmode_p2d2  long      _CLKMODE2                       '00008  clockmode for P2D2
clkmode_p2ev  long      _CLKMODEV                       '0000C  clockmode for P2EVAL
bitperx       long      _BITPER                         '00010  bit period to load into smartpins
clkmodex      long      0                               '00014  will store clkmode
clkfreqx      long      _CLOCKFREQ                      '00018  will store clkfreq
baudx         long      _BAUD                           '0001C  will store baud
'+-----------------------------------------------------------------------------+


'+-------[ check for 10K pull-down on pincfg ]---------------------------------+ 
begin         drvh      #pincfg                         '\ drive pin hi
              waitx     #30*1                           '| wait >1us
              flth      #pincfg                         '| float pin
              waitx     #30*5                           '| wait >5us
              testp     #pincfg                     wc  '/ P2D2=nc=pull-dn
              
              mov       pcb,              #0            ' P2D2: preset = 0
              mov       clkmodex,         clkmode_p2d2  ' P2D2: preset val
        if_c  mov       pcb,              #1            ' P2EVAL: set  = 1
        if_c  mov       clkmodex,         clkmode_p2ev  ' P2EVAL: set val
              mov       clkfreqx,         ##_CLOCKFREQ  ' get clock as constant
              
'+-------[ Set Xtal for P2D2 ]-------------------------------------------------+ 
              rdlong    lmm_x,            #$14      wz  ' if 0 skip pll fixup
        if_z  jmp       #.skip 
              andn      lmm_x,            #_XSEL        ' remove enable bit just in case
              hubset    lmm_x                           ' ... and disable
              waitx     ##20_000_000/30                 ' ... ~33ms 
.skip         wrlong    clkmodex,         #$14          ' write to hub $00014
              wrlong    clkfreqx,         #$18          ' write to hub $00018
              wrlong    baudx,            #$1C          ' write to hub $0001C
              hubset    clkmodex                        ' setup oscillator
              waitx     ##20_000_000/100                ' ... ~10ms
              or        clkmodex,         #_XSEL        ' ... add enable bits for XI+PLL
              hubset    clkmodex                        ' ... enable oscillator

'+-------[ Start Serial for P2D2 ]---------------------------------------------+ 
              mov       lmm_bufad,        ##_HUBBUF     ' locn of hub buffer for serial routine 
              mov       lmm_x,            ##_bitper     ' sets serial baud
              call      #_SerialInit                    ' initialise serial
'+-----------------------------------------------------------------------------+

loop          cmp       pcb,              #0        wz  ' z=P2D2 else P2EVAL

        if_z  drvnot    #LED_p2d2                       ' flash P2D2 green led
        if_nz drvnot    #LED_p2eval                     ' flash P2EVAL blue led

        if_z  mov       lmm_p,            ##str_p2d2    ' must be in hub!
        if_nz mov       lmm_p,            ##str_p2eval  ' must be in hub!
              call      #_HubTxString
              
              waitx     ##_CLOCKFREQ/1                  ' 1s
              jmp       #loop

'+-----------------------------------------------------------------------------+
              fit       $1E0                            ' don't overwrite ROM Monitor variables
'+-----------------------------------------------------------------------------+

''============[ HUB VARIABLES ]=================================================                                    
DAT           orgh      $4000
str_p2d2      byte      "Auto Detected P2D2",13,10,0
str_p2eval    byte      "Auto Detected P2EVAL",13,10,0
              alignl                                                                                                     
''---------------------------------------------------------------------------------------------------
'